# -*- coding: utf-8 -*-
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.urls import reverse
from django.db.models import Max

from lib.access_auth import access_frappe
from lib.tools import confirmation
from frappe.models import (
    Annee,
    Filiere,
    Matiere,
    Document,
    DocumentFichier,
    Commentaire,
)
from frappe.forms import CreateDocumentForm, EditDocumentForm, CommentaireForm
import re


@access_frappe
def index(request):
    c = {"annees": Annee.objects.filter(ecole=request.ecole).order_by("nom")}
    return render(request, "frappe/index.html", c)


@access_frappe
def mes_documents(request):
    documents = Document.objects.filter(auteur=request.user, matiere__filiere__annee__ecole=request.ecole).order_by(
        "matiere__filiere__annee__nom", "matiere__filiere__nom", "matiere__nom", "nom"
    )
    return render(request, "frappe/mes_documents.html", {"documents": documents})


@access_frappe
def editer_document(request, id):
    document = get_object_or_404(
        Document.objects.filter(auteur=request.user, matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )
    if request.method == "POST":
        form = EditDocumentForm(request.POST, request.FILES, instance=document, ecole=request.ecole)
        if form.is_valid():
            form.save()

            if form.cleaned_data["fichier"]:
                # on va le placer à la fin, on recupère l'ordre maximum
                ordre = document.fichiers().aggregate(Max("ordre"))["ordre__max"] + 1
                DocumentFichier(document=document, fichier=form.cleaned_data["fichier"], ordre=ordre).save()

            messages.success(request, "Document modifié avec succès.")
            return redirect("frappe:mes_documents")
    else:
        form = EditDocumentForm(instance=document, ecole=request.ecole)

    c = {
        "document": document,
        "form": form,
    }
    return render(request, "frappe/editer_document.html", c)


@access_frappe
def creer_document(request):
    if request.method == "POST":
        form = CreateDocumentForm(request.POST, request.FILES, ecole=request.ecole)
        if form.is_valid():
            document = form.save(commit=False)
            document.auteur = request.user
            document.save()
            form.save_m2m()
            DocumentFichier(document=document, fichier=form.cleaned_data["fichier"], ordre=0).save()
            messages.success(request, "Document créé avec succès.")
            return redirect("frappe:show_matiere", id=document.matiere.pk)
    else:
        matiere_id = request.GET.get("matiere")
        if matiere_id and re.match("^[0-9]+$", matiere_id):
            matiere = get_object_or_404(Matiere, id=matiere_id)
        else:
            matiere = None
        if not Matiere.objects.filter(filiere__annee__ecole=request.ecole).exists():
            messages.error(request, "Veuillez créer une matière.")
            page_precedente = request.META.get("HTTP_REFERER")
            if page_precedente is None:
                page_precedente = reverse("frappe:index")
            return redirect(page_precedente)

        form = CreateDocumentForm(ecole=request.ecole, initial={"matiere": matiere})

    return render(request, "frappe/creer_document.html", {"form": form})


@access_frappe
def supprimer_document(request, id):
    document = get_object_or_404(
        Document.objects.filter(auteur=request.user, matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Document supprimé avec succès.")
            document.delete()

        return redirect("frappe:mes_documents")

    message = f"Voulez-vous vraiment supprimer le document « {document.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@access_frappe
def docfichiers_supprimer(request, id):
    docfichier = get_object_or_404(
        DocumentFichier.objects.filter(
            document__auteur=request.user,
            document__matiere__filiere__annee__ecole=request.ecole,
        ),
        pk=id,
    )

    if docfichier.document.fichiers().count() == 1:
        messages.error(request, "Vous devriez plutôt supprimer le document entier.")
        return redirect("frappe:editer_document", docfichier.document.pk)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Fichier supprimé avec succès.")
            docfichier.delete()

        return redirect("frappe:editer_document", docfichier.document.pk)

    message = f"Voulez-vous vraiment supprimer le fichier « {docfichier} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


@access_frappe
def docfichiers_up(request, id):
    docfichier = get_object_or_404(
        DocumentFichier.objects.filter(
            document__auteur=request.user,
            document__matiere__filiere__annee__ecole=request.ecole,
        ),
        pk=id,
    )

    # on recupère le fichier placé au dessus
    docfichiers = docfichier.document.fichiers().filter(ordre__lt=docfichier.ordre).order_by("-ordre")

    if docfichiers.count() > 0:
        docfichier_upper = docfichiers[0]
        # on inverse les ordres
        docfichier.ordre, docfichier_upper.ordre = (
            docfichier_upper.ordre,
            docfichier.ordre,
        )
        docfichier.save()
        docfichier_upper.save()

    messages.success(request, "Ordre modifié avec succès.")
    return redirect("frappe:editer_document", docfichier.document.pk)


@access_frappe
def docfichiers_down(request, id):
    docfichier = get_object_or_404(
        DocumentFichier.objects.filter(
            document__auteur=request.user,
            document__matiere__filiere__annee__ecole=request.ecole,
        ),
        pk=id,
    )

    # on recupère le fichier placé en dessous
    docfichiers = docfichier.document.fichiers().filter(ordre__gt=docfichier.ordre).order_by("ordre")

    if docfichiers.count() > 0:
        docfichier_upper = docfichiers[0]
        # on inverse les ordres
        docfichier.ordre, docfichier_upper.ordre = (
            docfichier_upper.ordre,
            docfichier.ordre,
        )
        docfichier.save()
        docfichier_upper.save()

    messages.success(request, "Ordre modifié avec succès.")
    return redirect("frappe:editer_document", docfichier.document.pk)


@access_frappe
def show_annee(request, id):
    annee = get_object_or_404(Annee.objects.filter(ecole=request.ecole), pk=id)
    return render(request, "frappe/show_annee.html", {"annee": annee})


@access_frappe
def show_filiere(request, id):
    filiere = get_object_or_404(Filiere.objects.filter(annee__ecole=request.ecole), pk=id)
    return render(request, "frappe/show_filiere.html", {"filiere": filiere})


@access_frappe
def show_matiere(request, id):
    matiere = get_object_or_404(Matiere.objects.filter(filiere__annee__ecole=request.ecole), pk=id)
    matiere.documents = matiere.documents()
    for document in matiere.documents:
        document.commentaire_form = CommentaireForm()

    return render(request, "frappe/show_matiere.html", {"matiere": matiere})


@access_frappe
def poster_commentaire(request, id):
    document = get_object_or_404(Document.objects.filter(matiere__filiere__annee__ecole=request.ecole), pk=id)

    if request.method == "POST":
        form = CommentaireForm(request.POST)
        if form.is_valid():
            commentaire = form.save(commit=False)
            commentaire.document = document
            commentaire.auteur = request.user
            commentaire.save()
            form.save_m2m()
            messages.success(request, "Commentaire ajouté avec succès.")
            return redirect("frappe:show_matiere", document.matiere.pk)
    else:
        form = CommentaireForm()

    c = {
        "document": document,
        "form": form,
    }
    return render(request, "frappe/poster_commentaire.html", c)


@access_frappe
def editer_commentaire(request, id):
    commentaire = get_object_or_404(
        Commentaire.objects.filter(document__matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )

    if request.method == "POST":
        form = CommentaireForm(request.POST, instance=commentaire)
        if form.is_valid():
            form.save()
            messages.success(request, "Commentaire modifié avec succès.")
            return redirect("frappe:show_matiere", commentaire.document.matiere.pk)
    else:
        form = CommentaireForm(instance=commentaire)

    c = {
        "commentaire": commentaire,
        "form": form,
    }
    return render(request, "frappe/editer_commentaire.html", c)


@access_frappe
def supprimer_commentaire(request, id):
    commentaire = get_object_or_404(
        Commentaire.objects.filter(document__matiere__filiere__annee__ecole=request.ecole),
        pk=id,
    )

    if request.method == "POST":
        if "conf_ok" in request.POST:
            messages.success(request, "Commentaire supprimé avec succès.")
            commentaire.delete()

        return redirect("frappe:show_matiere", commentaire.document.matiere.pk)

    message = f"Voulez-vous vraiment supprimer votre commentaire sur le document « {commentaire.document.nom} » ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)
